# 16주 완성! AI 전문가 양성 과정: 파이썬부터 프로덕션 배포까지

![Course Banner](./assets/course_banner.png)

[![Python](https://img.shields.io/badge/Python-3.10+-blue.svg)](https://www.python.org/downloads/)
[![Poetry](https://img.shields.io/badge/Poetry-dependency%20management-blue.svg)](https://python-poetry.org/)
[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)
[![Ruff](https://img.shields.io/badge/linter-ruff-red.svg)](https://github.com/astral-sh/ruff)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
[![CI/CD](https://img.shields.io/badge/CI%2FCD-GitLab%20CI-orange.svg)](http://gitlab.geumdo.net/root/geumdo_docs/-/pipelines)

"아이디어는 있는데, 어떻게 만들어야 할지 막막하신가요?"
"AI, 이제는 소비하는 것을 넘어 직접 만들고 싶으신가요?"

본 과정은 파이썬 기초 문법부터 시작하여, 실제 동작하는 AI 서비스를 직접 기획하고, 개발하며, 세상에 배포하는 전 과정을 경험하는 **16주 완성 실전형 AI 엔지니어링 코스**입니다. 막연한 아이디어를 현실로 만드는 '만들기 능력자'로 거듭나세요.

---

## 🚀 이 과정을 통해 얻을 수 있는 것

- **AI 서비스 개발의 A to Z**: 데이터 분석, 머신러닝/딥러닝 모델링, API 서버 개발, Docker를 이용한 배포까지, AI 제품 개발의 전체 사이클을 직접 경험하고 이해합니다.
- **실무 중심의 프로젝트 경험**: 체계적인 학습 후, '나만의 개성이 담긴 **캡스톤 프로젝트**'를 진행하며 현업 수준의 문제 해결 능력과 포트폴리오를 확보합니다.
- **생산성을 극대화하는 최신 기술 스택**: `FastAPI`, `Docker`, `PyTorch`, `LangChain` 등 현업에서 가장 수요가 높은 기술 스택을 사용합니다. `Cursor`, `GitHub Copilot` 같은 AI 네이티브 도구를 활용하여 개발 효율을 극대화하는 방법을 체득합니다.
- **지속 가능한 성장을 위한 발판**: MLOps의 기본 개념을 이해하고, 최신 AI 기술 트렌드를 읽는 눈을 기르며, AI 전문가로서 지속적으로 성장할 수 있는 학습 로드맵을 얻게 됩니다.

---

## 📚 전체 커리큘럼 개요

| 파트 (Part) | 주차 (Week) | 주제 (Topic) | 관련 문서 | 소스코드 |
|:---:|:---:|:---|:---|:---|
| **1** | 1-5주차 | **AI 개발을 위한 파이썬 마스터리** | [0. Intro](./courses/00_introduction/part_0_introduction.md)<br/>[0.1. History](./courses/00_introduction/part_0.1_history_of_ai.md)<br/>[1. Environment](./courses/01_ai_development_environment/part_1_ai_development_environment.md)<br/>[2. Python Core](./courses/02_python_core_syntax/part_2_python_core_syntax.md)<br/>[3. Collections](./courses/03_python_collections/part_3_python_collections.md)<br/>[4. OOP](./courses/04_object_oriented_programming/part_4_object_oriented_programming.md)<br/>[5. AI Libs](./courses/05_ai_core_libraries/part_5_ai_core_libraries.md) | [Part 2](./source_code/02_python_core_syntax/)<br/>[Part 3](./source_code/03_python_collections/)<br/>[Part 4](./source_code/04_object_oriented_programming/)<br/>[Part 5](./source_code/05_ai_core_libraries/) |
| **2** | 6-9주차 | **핵심 AI 모델 개발 및 서빙** | [6. ML](./courses/06_machine_learning/part_6_machine_learning.md)<br/>[7. DL](./courses/07_deep_learning/part_7_deep_learning.md)<br/>[7.1. RNN](./courses/07_deep_learning/part_7.1_recurrent_neural_networks.md)<br/>[7.2. Transformer](./courses/07_deep_learning/part_7.2_transformer_and_llm_principles.md)<br/>[7.3. LangChain](./courses/07_deep_learning/part_7.3_llm_application_development_with_langchain.md)<br/>[7.4. GNN](./courses/07_deep_learning/part_7.4_graph_neural_networks.md)<br/>[7.5. RL](./courses/07_deep_learning/part_7.5_reinforcement_learning.md)<br/>[8. FastAPI Serving](./courses/08_model_serving_with_fastapi/part_8_model_serving_with_fastapi.md)<br/>[9. Production API](./courses/09_production_ready_api/part_9_production_ready_api.md) | [Part 6](./source_code/06_machine_learning/)<br/>[Part 7](./source_code/07_deep_learning/)<br/>[Part 8](./source_code/08_model_serving_with_fastapi/)<br/>[Part 9](./source_code/09_production_ready_api/) |
| **3** | 10-13주차 | **프로덕션 MLOps 및 AI 심화** | [10. Expert Path](./courses/10_expert_path/part_10_expert_path.md)<br/>[11. MLOps](./courses/11_mlops/part_11_mlops.md)<br/>[12. Optimization](./courses/12_model_optimization/part_12_model_optimization.md)<br/>[13. Generative AI](./courses/13_generative_ai/part_13_generative_ai.md)<br/>[14. AI Ethics](./courses/14_ai_ethics/part_14_ai_ethics.md) | [Part 11](./source_code/11_mlops/)<br/>[Part 12](./source_code/12_model_optimization/)<br/>[Part 13](./source_code/13_generative_ai/)<br/>[Part 14](./source_code/14_ai_ethics/) |
| **4** | 14-15주차 | **최종 캡스톤 프로젝트** | [15. Capstone](./courses/15_capstone_project/part_15_capstone_project.md) | (프로젝트 개별 진행) |

---

## 🛠️ 기술 스택 (Tech Stack)

- **언어**: Python 3.10+
- **의존성 관리**: Poetry (권장) / pip
- **핵심 라이브러리**: NumPy, Pandas, Scikit-learn, PyTorch, LangChain
- **API 서버**: FastAPI
- **배포 및 운영**: Docker, Git
- **개발 환경**: JupyterLab, Visual Studio Code, Cursor, GitHub Copilot
- **코드 품질**: Black, Ruff, MyPy, Pytest
- **CI/CD**: GitLab CI/CD

---

## 📖 학습 자료

- **[개발 환경 설정 가이드 (SETUP.md)](./SETUP.md)**: Poetry를 사용한 현대적인 개발 환경 설정 방법
- **[전체 강의 계획서 (Course Schedule)](./AI_Expert_Course_Schedule.md)**: 16주 과정의 상세한 주차별 학습 목표와 내용을 확인하세요.
- **[파트별 강의 노트 및 용어집 (Courses)](./courses/)**: 각 파트별 강의 자료와 핵심 용어집을 확인할 수 있습니다.
- **[실습 코드 모음 (Source Code)](./source_code/)**: Jupyter Notebook과 Python 파일로 구성된 실습 코드를 직접 실행하고 수정해볼 수 있습니다.
- **[통합 용어집 (Glossary)](./glossary.md)**: 전체 과정에서 사용되는 핵심 용어들을 한 곳에서 확인할 수 있습니다.

> [!NOTE]
> 이 과정은 '금도'님의 AI 강의 자료를 기반으로, 수강생의 학습 경험 극대화를 위해 커리큘럼을 재구성하고 내용을 심화시킨 결과물입니다. 원작자의 훌륭한 비유와 설명에 깊이 감사합니다. 