# Part 2: 파이썬 핵심 문법 - 핵심 용어집

- **변수 (Variable)**
  - **정의**: 특정 값을 저장하기 위해 이름을 붙인 메모리 공간. 변수에 저장된 값은 프로그램 실행 중에 변경될 수 있습니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **자료형 (Data Type)**
  - **정의**: 프로그래밍 언어에서 다룰 수 있는 데이터의 종류. 파이썬의 기본 자료형에는 숫자형(`int`, `float`), 문자열(`str`), 불리언(`bool`) 등이 있습니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **숫자형 (Numeric Types)**
  - **정의**: 숫자를 나타내는 자료형.
    - **정수 (Integer, `int`)**: `1`, `100`, `-5` 같이 소수점이 없는 숫자.
    - **실수 (Float)**: `3.14`, `-0.5` 같이 소수점이 있는 숫자.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **문자열 (String, `str`)**
  - **정의**: `'Hello'`, `"Python"`과 같이 따옴표로 감싼 글자들의 나열(시퀀스). 텍스트 데이터를 표현하는 데 사용됩니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **불리언 (Boolean, `bool`)**
  - **정의**: `True`(참)와 `False`(거짓) 두 가지 값만 가지는 자료형. 주로 조건문의 결과를 표현하는 데 사용됩니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **리스트 (List)**
  - **정의**: 여러 개의 값을 순서대로 저장하는 파이썬의 대표적인 자료구조. 대괄호(`[]`)를 사용하여 생성하며, 저장된 값(요소)들은 변경 가능(mutable)하고, 중복된 값을 허용합니다. 인덱스를 통해 각 요소에 접근할 수 있습니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md), [Part 3: 파이썬 컬렉션 심화](../03_python_collections/part_3_python_collections.md)

- **튜플 (Tuple)**
  - **정의**: 여러 개의 값을 순서대로 저장하는 파이썬의 대표적인 자료구조. 소괄호(`()`)를 사용하여 생성하며, 저장된 값(요소)들은 변경 불가능(immutable)합니다. 인덱스를 통해 각 요소에 접근할 수 있습니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md), [Part 3: 파이썬 컬렉션 심화](../03_python_collections/part_3_python_collections.md)

- **딕셔너리 (Dictionary, `dict`)**
  - **정의**: 키(Key)와 값(Value)을 하나의 쌍으로 묶어 저장하는 파이썬의 자료구조. 각 키는 고유해야 하며, 키를 통해 해당 값을 빠르게 찾거나 수정, 삭제할 수 있습니다. 중괄호(`{}`)를 사용하여 생성하며, 데이터의 순서가 중요하지 않고 의미 있는 이름으로 데이터에 접근하고 싶을 때 유용합니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md), [Part 3: 파이썬 컬렉션 심화](../03_python_collections/part_3_python_collections.md)

- **셋 (Set)**
  - **정의**: 중복을 허용하지 않는, 순서 없는 요소들의 모음. 중괄호(`{}`)나 `set()` 함수로 생성합니다. 주로 데이터의 중복을 제거하거나, 멤버십 테스트(특정 요소의 존재 여부 확인), 집합 연산(합집합, 교집합 등)에 사용됩니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **제어문 (Control Flow Statement)**
  - **정의**: 프로그램의 실행 흐름을 특정 조건이나 반복에 따라 제어하는 구문.
    - **조건문 (`if`, `elif`, `else`)**: 주어진 조건의 참/거짓에 따라 다른 코드 블록을 실행합니다.
    - **반복문 (`for`, `while`)**: 특정 코드 블록을 여러 번 반복해서 실행합니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **들여쓰기 (Indentation)**
  - **정의**: 파이썬에서 코드 블록의 시작과 끝을 나타내는 문법적 요소. 다른 언어들이 중괄호(`{}`) 등을 사용하는 것과 달리, 파이썬은 동일한 수의 공백(일반적으로 스페이스 4칸)을 사용하여 코드의 논리적 구조와 종속 관계를 표현합니다. 올바른 들여쓰기는 파이썬 코드의 가독성을 높이고 문법 오류를 방지하는 데 매우 중요합니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **함수 (Function)**
  - **정의**: 특정 작업을 수행하는 코드 블록에 이름을 붙인 것. 코드를 재사용하고, 프로그램을 모듈화하여 구조를 더 명확하게 만드는 데 사용됩니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **매개변수 (Parameter)와 인자 (Argument)**
  - **정의**:
    - **매개변수 (Parameter)**: 함수를 정의할 때, 함수가 호출될 때 전달받을 값을 담기 위해 선언하는 변수 이름. 함수의 '입력 인터페이스' 역할을 합니다.
    - **인자 (Argument)**: 함수를 호출할 때, 매개변수에 실제로 전달되는 값.
  - **예시**: `def add(a, b): # a, b는 매개변수` / `add(3, 5) # 3, 5는 인자`
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **반환 값 (Return Value)**
  - **정의**: 함수가 `return` 키워드를 사용하여 자신의 실행 결과를 호출한 쪽으로 돌려주는 값. 함수는 특정 계산을 수행한 뒤 그 결과를 반환하여 다른 변수에 저장하거나 다른 연산에 사용될 수 있게 합니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **리스트 컴프리헨션 (List Comprehension)**
  - **정의**: 기존 리스트나 다른 순회 가능한(iterable) 객체로부터 새로운 리스트를 간결하고 효율적으로 생성하는 파이썬의 기능. `for` 반복문과 `if` 조건문을 한 줄로 표현하여 가독성 높고 파이썬다운(Pythonic) 코드를 작성할 수 있게 합니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md)

- **슬라이싱 (Slicing)**
  - **정의**: 리스트, 튜플, 문자열 등 순서가 있는 시퀀스 자료형의 일부를 잘라내어 새로운 시퀀스를 만드는 기능. `시퀀스[시작:끝:간격]` 형태로 사용합니다.
  - **관련 파트**: [Part 2: 파이썬 핵심 문법](./part_2_python_core_syntax.md) 