# Part 2: 파이썬 핵심 문법 마스터하기

**⬅️ 이전 시간: [Part 1: AI 개발 환경 완벽 구축 가이드](../01_ai_development_environment/part_1_ai_development_environment.md)**
**➡️ 다음 시간: [Part 3: 파이썬 컬렉션, 더 깊게 이해하기](../03_python_collections/part_3_python_collections.md)**

---

## 1. 학습 목표 (Learning Objectives)

이번 파트가 끝나면, 여러분은 다음을 할 수 있게 됩니다.

- 변수를 선언하고 숫자, 문자열, 불리언 등 기본 자료형을 활용할 수 있습니다.
- [리스트](./glossary.md#리스트-list), [튜플](./glossary.md#튜플-tuple), [딕셔너리](./glossary.md#딕셔너리-dictionary), 셋의 차이점을 이해하고 상황에 맞게 선택하여 사용할 수 있습니다.
- `if`, `for`, `while` 등 제어문을 사용하여 코드의 실행 흐름을 제어할 수 있습니다.
- 코드의 재사용성을 높이는 [함수](./glossary.md#함수-function)를 직접 정의하고, [매개변수](./glossary.md#매개변수-parameter와-인자-argument)와 [반환 값](./glossary.md#반환-값-return-value)을 활용할 수 있습니다.
- [리스트 컴프리헨션](./glossary.md#리스트-컴프리헨션-list-comprehension)과 같은 파이썬다운(Pythonic) 코드를 작성할 수 있습니다.

## 2. 핵심 키워드 (Keywords)

`변수(Variable)`, `[자료형(Data Type)](./glossary.md#자료구조-data-structure)`, `[리스트(List)](./glossary.md#리스트-list)`, `[튜플(Tuple)](./glossary.md#튜플-tuple)`, `[딕셔너리(Dictionary)](./glossary.md#딕셔너리-dictionary)`, `셋(Set)`, `제어문(Control Flow)`, `[들여쓰기(Indentation)](./glossary.md#들여쓰기-indentation)`, `[함수(Function)](./glossary.md#함수-function)`

## 3. 왜 파이썬 문법이 중요할까요? (Introduction)

AI 개발의 세계에 오신 것을 환영합니다! 파이썬은 AI와 데이터 과학 분야에서 가장 사랑받는 언어입니다. 문법이 간결하고 사람의 생각과 비슷해서, 프로그래밍을 처음 시작하는 분들도 쉽게 배울 수 있습니다. 이번 주차에는 앞으로 우리가 만들 모든 AI 서비스의 뼈대가 될 파이썬의 가장 핵심적인 문법들을 하나씩 정복해나갑니다.

> [!TIP]
> 이 파트의 모든 코드 예제는 아래 링크의 파이썬 파일에서 직접 실행하고 수정해볼 수 있습니다.
> **실행 가능한 소스 코드: [`part_2_3_python_syntax_collections.py`](../../source_code/part_2_3_python_syntax_collections.py)**

---

## 4. 모든 것의 시작, 변수와 자료형

> **🎯 1일차 목표:** 변수(Variable)를 사용해 데이터를 저장하고, 파이썬의 기본 데이터 종류(자료형)를 이해합니다.

### 4.1. 변수(Variable): 데이터에 이름표 붙이기
<a href="./glossary.md#변수-variable">변수(Variable)</a>는 숫자, 글자 같은 데이터를 저장하는 '상자'입니다. `age = 30` 이라는 코드는, `30`이라는 데이터를 `age`라는 이름의 상자에 넣는다는 뜻입니다. 이제부터 우리는 `age`라는 이름만 부르면 컴퓨터가 알아서 `30`이라는 값을 찾아줍니다.

```python
my_name = "파이"      # 문자열 (String)
my_age = 20         # 정수 (Integer)
pi = 3.14           # 실수 (Float)
is_student = True   # 불리언 (Boolean)

print(f"안녕하세요, 제 이름은 {my_name}이고 나이는 {my_age}살입니다.")
```

### 4.2. 기본 자료형(Data Types)
파이썬은 데이터의 종류를 구분하며, 이를 <a href="./glossary.md#자료구조-data-structure">자료형</a>이라고 합니다.
- **정수 (Integer)**: `1`, `100`, `-5` 같이 소수점이 없는 숫자.
- **실수 (Float)**: `3.14`, `-0.5` 같이 소수점이 있는 숫자.
- **문자열 (String)****: `"안녕하세요"`, `'Python'` 같이 따옴표로 감싼 글자들.
- **<a href="./glossary.md#boolean">불리언 (Boolean)</a>**: `True` 또는 `False`. 조건의 참/거짓을 나타냅니다.

---

## 5. 여러 데이터를 담는 그릇 (1): 리스트와 튜플

> **🎯 2일차 목표:** 순서가 있는 데이터 묶음인 [리스트(List)](./glossary.md#리스트-list)와 변경이 불가능한 [튜플(Tuple)](./glossary.md#튜플-tuple)을 다루고, [리스트 컴프리헨션](./glossary.md#리스트-컴프리헨션-list-comprehension)으로 파이썬다운 코드를 작성합니다.

### 5.1. 리스트(List): 순서가 있고 변경 가능한 만능 주머니
- 여러 값을 순서대로 저장하는 **변경 가능한** <a href="./glossary.md#자료구조-data-structure">자료구조</a>입니다. (`[]` 사용)

```python
fruits = ["apple", "banana", "cherry"]
fruits[1] = "blueberry"      # 값 변경
fruits.append("strawberry")  # 요소 추가
print(f"최종 리스트: {fruits}")
```

### 5.2. 파이썬다운 코드: 리스트 컴프리헨션 (List Comprehension)
`for` 반복문을 한 줄로 간결하게 표현하여 새로운 리스트를 만드는 방법입니다.

```python
# 0부터 9까지의 숫자 중 짝수만 제곱하여 리스트 만들기
squares = [i**2 for i in range(10) if i % 2 == 0]
print(f"짝수의 제곱 리스트: {squares}") # [0, 4, 16, 36, 64]
```

### 5.3. 튜플(Tuple): 순서가 있지만 변경 불가능한 금고
- 여러 값을 순서대로 저장하는 **변경 불가능한** <a href="./glossary.md#자료구조-data-structure">자료구조</a>입니다. (`()` 사용)
- 한 번 만들어지면 내용을 바꿀 수 없어 안정성이 중요할 때 (e.g., 함수의 반환 값, 좌표) 사용됩니다.

```python
point = (10, 20)
# point[0] = 15 #! TypeError 발생
```

---

## 6. 여러 데이터를 담는 그릇 (2): 딕셔너리와 셋

> **🎯 3일차 목표:** `Key:Value` 쌍으로 데이터를 저장하는 [딕셔너리(Dictionary)](./glossary.md#딕셔너리-dictionary)와 중복을 허용하지 않는 셋(Set)을 다룹니다.

### 6.1. 딕셔너리(Dictionary): 의미를 부여한 데이터 관리법
- `Key:Value` 쌍으로 데이터를 저장하며, 순서보다 '의미(Key)'를 통해 값을 찾는 데 특화된 <a href="./glossary.md#자료구조-data-structure">자료구조</a>입니다. (`{}` 사용)

```python
user = {"name": "앨리스", "age": 30}
user['email'] = "alice@example.com" # 데이터 추가/변경
print(f"이름: {user['name']}, 이메일: {user.get('email')}")
```

### 6.2. 셋(Set): 중복을 제거하고 관계를 파악하는 도구
- **중복을 허용하지 않는** 순서 없는 데이터의 모음입니다. (`{}` 사용)
- 데이터의 중복 제거, 그룹 간의 합집합, 교집합 등을 구할 때 유용한 <a href="./glossary.md#set">자료구조</a>입니다.

```python
unique_numbers = {1, 2, 3, 2, 1, 4}
print(f"중복이 제거된 집합: {unique_numbers}") # {1, 2, 3, 4}

set_a = {1, 2, 3}
set_b = {3, 4, 5}
print(f"교집합: {set_a & set_b}") # {3}
```

---

## 7. 코드의 흐름 제어하기

> **🎯 4일차 목표:** `if`, `for`, `while` 제어문을 사용하여 조건과 반복에 따라 코드의 흐름을 제어합니다.

### 7.1. 파이썬의 상징: 들여쓰기(Indentation)
파이썬은 **[들여쓰기](./glossary.md#들여쓰기-indentation)(스페이스 4칸)**로 코드의 소속을 구분하는 '문법'입니다. `if`, `for` 등에 속한 코드들은 반드시 들여쓰기를 해야 합니다.

### 7.2. `if-elif-else`: 조건에 따라 움직이기
```python
score = 85
if score >= 90: grade = "A"
elif score >= 80: grade = "B"
else: grade = "F"
print(f"학점: {grade}")
```

### 7.3. `for`: 순서대로 반복하기
리스트, 튜플, 문자열 등 순회 가능한 객체의 요소를 하나씩 순회합니다. `enumerate`를 사용하면 인덱스와 값을 함께 얻을 수 있습니다.

```python
fruits = ["apple", "banana", "cherry"]
for idx, fruit in enumerate(fruits):
    print(f"{idx+1}번째 과일: {fruit}")
```

### 7.4. `while`: 조건이 만족하는 동안 반복하기
주어진 조건이 `True`인 동안 코드를 계속해서 반복합니다. 조건이 언젠가 `False`가 되도록 만들지 않으면 '무한 루프'에 빠지므로 주의해야 합니다.

```python
count = 3
while count > 0:
    print(f"카운트 다운: {count}")
    count -= 1 # 이 줄이 없으면 무한 루프!
print("발사!")
```

---

## 8. 코드를 재사용하는 마법, 함수

> **🎯 5일차 목표:** 반복되는 코드를 `[함수(Function)](./glossary.md#함수-function)`로 묶어 재사용하고, 함수의 입력([매개변수](./glossary.md#매개변수-parameter와-인자-argument))과 출력([반환 값](./glossary.md#반환-값-return-value))을 이해합니다.

### 8.1. 함수(Function): 나만의 미니 프로그램
[함수](./glossary.md#함수-function)는 특정 작업을 수행하는 코드 묶음에 이름을 붙인 **'재사용 가능한 레시피'**입니다. `def` 키워드로 정의하고, 이름으로 호출하여 사용합니다.

### 8.2. 입력(매개변수)과 출력(반환 값)
- **[매개변수(Parameter)](./glossary.md#매개변수-parameter와-인자-argument):** 함수에 전달하는 재료(입력 값).
- **[`return`](./glossary.md#반환-값-return-value):** 함수가 작업을 마친 후 돌려주는 결과물(출력 값).

```python
def add(a, b):
    result = a + b
    return result

# 함수 호출 및 결과 저장
sum_result = add(5, 3)
print(f"5 + 3 = {sum_result}")
```
함수를 잘 사용하면 복잡한 문제를 여러 개의 간단한 함수로 나누어 해결할 수 있어, 훨씬 체계적인 프로그래밍이 가능해집니다.

---

## 9. 직접 해보기 (Hands-on Lab)

이번 주에 배운 내용을 바탕으로 다음 함수들을 직접 완성해보세요.

**문제 1: 학점 계산기 함수**
- 학생의 점수(0~100)를 매개변수로 받아 학점을 반환하는 `get_grade` 함수를 `if-elif-else`를 사용하여 만들어보세요.
  - 90점 이상: "A", 80점 이상: "B", 70점 이상: "C", 70점 미만: "F"

**문제 2: 리스트에서 특정 단어 개수 세기**
- 과일 이름 리스트와 특정 과일 이름을 매개변수로 받아, 리스트 안에 그 과일이 몇 번 나타나는지 `for`문을 사용해 세고 개수를 반환하는 `count_word` 함수를 만들어보세요.

**문제 3: 구구단 출력 함수**
- 숫자를 하나 매개변수로 받아 해당 숫자의 구구단을 `while`문을 사용하여 출력하는 `multiplication_table` 함수를 만들어보세요. (이 함수는 값을 반환할 필요가 없습니다.)

## 10. 되짚어보기 (Summary)

이번 주차에 우리는 파이썬의 가장 기본적인 구성 요소들을 배웠습니다.

- **변수와 자료형**: 데이터를 저장하고(변수), 데이터의 종류(숫자, 문자열, 불리언)를 구분했습니다.
- **자료구조**: 여러 데이터를 담는 그릇인 리스트, 튜플, 딕셔너리, 셋을 익혔습니다.
- **제어문**: `if`, `for`, `while`을 통해 코드의 흐름을 제어하는 방법을 배웠습니다.
- **함수**: `def`를 사용해 반복되는 코드를 재사용 가능한 블록으로 만들었습니다.

이 핵심 문법들은 앞으로 우리가 만들 모든 AI 서비스의 뼈대가 될 것입니다.

## 11. 더 깊이 알아보기 (Further Reading)

- [파이썬 공식 튜토리얼: 자료구조 (리스트, 튜플, 셋, 딕셔너리)](https://docs.python.org/ko/3/tutorial/datastructures.html)
- [파이썬 공식 튜토리얼: 제어 흐름 도구 (if, for, while)](https://docs.python.org/ko/3/tutorial/controlflow.html)
- [파이썬 공식 튜토리얼: 함수 정의하기](https://docs.python.org/ko/3/tutorial/controlflow.html#defining-functions)