# Part 7.1: 순환 신경망 (RNN)과 LSTM

**⬅️ 이전 시간: [Part 7: 딥러닝 기초와 PyTorch](./part_7_deep_learning.md)**
**➡️ 다음 시간: [Part 7.2: Transformer와 LLM의 핵심 원리](./part_7.2_transformer_and_llm_principles.md)**

---

## 1. 학습 목표 (Learning Objectives)

이번 파트가 끝나면, 여러분은 다음을 할 수 있게 됩니다.

- 순차 데이터(Sequential Data)의 특징을 이해하고 RNN이 왜 필요한지 설명할 수 있습니다.
- RNN의 기본 구조와 순환하는 아이디어, 그리고 은닉 상태(Hidden State)의 역할을 이해합니다.
- 장기 의존성 문제(Vanishing/Exploding Gradient)가 무엇인지 설명할 수 있습니다.
- LSTM의 게이트(Gate) 구조가 어떻게 장기 의존성 문제를 해결하는지 기본 원리를 이해합니다.
- PyTorch를 사용하여 간단한 RNN/LSTM 모델을 구축하고 텍스트 생성 문제를 해결할 수 있습니다.

## 2. 핵심 키워드 (Keywords)

`순차 데이터(Sequential Data)`, `순환 신경망(RNN)`, `은닉 상태(Hidden State)`, `BPTT(Backpropagation Through Time)`, `장기 의존성 문제(Long-Term Dependency Problem)`, `기울기 소실/폭발(Vanishing/Exploding Gradient)`, `LSTM(Long Short-Term Memory)`, `게이트(Gate)`

## 3. 도입: 과거를 기억하는 모델, RNN

지금까지 우리가 다룬 MLP나 CNN은 입력 데이터 간의 순서나 시간적인 관계를 고려하지 않았습니다. 사진 속 고양이의 픽셀 위치는 중요하지만, '고양이'라는 단어를 이루는 '고', '양', '이' 글자들은 **순서**가 없다면 의미가 없습니다.

이렇게 **순서가 중요한 데이터**를 **순차 데이터(Sequential Data)**라고 합니다.
- **텍스트**: 단어의 순서가 문장의 의미를 결정합니다. ("나는 너를 좋아해" vs "너는 나를 좋아해")
- **시계열 데이터**: 주가, 날씨 등 시간의 흐름에 따라 기록된 데이터입니다. 어제의 주가가 오늘의 주가에 영향을 미칩니다.
- **음성**: 소리의 파형은 시간 순서대로 이어져 의미를 가집니다.

**순환 신경망(Recurrent Neural Network, RNN)**은 바로 이 순차 데이터를 처리하기 위해 탄생한 특별한 아키텍처입니다. RNN은 모델 내부에 '기억'을 할 수 있는 **순환(Recurrent)** 구조를 가지고 있어, 이전 시간 단계(time step)의 정보를 현재 계산에 활용할 수 있습니다.

---

## 4. RNN의 작동 원리

### 4-1. 순환과 은닉 상태 (Recurrence and Hidden State)

RNN의 핵심 아이디어는 간단합니다. 네트워크의 출력이 다시 자기 자신에게 입력으로 들어가는 것입니다.

![RNN Unrolled](https://i.imgur.com/4f2p7gH.png)
*(이미지 출처: Christopher Olah's Blog)*

왼쪽의 압축된 형태를 보면, RNN 계층은 입력(`x_t`)을 받아 출력(`h_t`)을 내보내고, 이 출력(`h_t`)이 다시 다음 계산을 위해 자기 자신에게 입력됩니다. 이 과정을 오른쪽처럼 시간의 흐름에 따라 펼쳐보면(Unrolled), 각 시간 단계(time step)별로 동일한 가중치를 공유하는 네트워크가 연결된 모습과 같습니다.

여기서 가장 중요한 개념은 **은닉 상태(Hidden State)**, 즉 `h_t` 입니다.

- **은닉 상태 (h_t)**: 해당 시간 단계 `t`까지의 '기억' 또는 '문맥'을 요약한 벡터입니다. 이 벡터는 이전 시간 단계의 은닉 상태(`h_t-1`)와 현재 시간 단계의 입력(`x_t`)을 함께 사용하여 계산됩니다.
- **수식**: `h_t = tanh(W_xh * x_t + W_hh * h_t-1 + b)`
    - `W_xh`: 입력 `x_t`에 대한 가중치
    - `W_hh`: 이전 은닉 상태 `h_t-1`에 대한 가중치
    - `b`: 편향(bias)
    - `tanh`: 활성화 함수 (주로 Tanh가 사용됨)

이 과정을 통해 RNN은 "나는", "너를" 이라는 단어를 순서대로 읽으며, "좋아해"라는 단어가 나올 차례라는 것을 예측할 수 있게 됩니다.

### 4-2. 시간 역전파 (BPTT)
RNN의 학습은 시간의 흐름에 따라 펼쳐진 네트워크를 따라 오차를 거꾸로 전파하는 **BPTT(Backpropagation Through Time)** 방식을 사용합니다. 이는 일반적인 역전파와 원리는 같지만, 모든 시간 단계에 걸쳐 역전파가 일어난다는 점이 다릅니다.

---

## 5. 장기 의존성 문제 (The Long-Term Dependency Problem)

RNN의 아이디어는 훌륭하지만, 치명적인 약점이 있습니다. 문장이 길어지면 **아주 먼 과거의 정보가 현재까지 전달되지 못하는 현상**, 즉 **장기 의존성 문제**가 발생합니다.

> "오늘 아침 [프랑스]에서 유창하게 [프랑스어]를 구사하는 사람들을 만났다."

위 문장에서 마지막 단어 '프랑스어'를 예측하려면, 문장 맨 앞의 '프랑스'라는 단어를 기억해야 합니다. 하지만 이 두 단어 사이의 거리가 너무 멀면, BPTT 과정에서 그래디언트(기울기)가 전달되다가 점점 희미해지거나( **기울기 소실, Vanishing Gradient** ), 반대로 너무 커져서 발산하는( **기울기 폭발, Exploding Gradient** ) 문제가 발생합니다.

- **기울기 소실**: 활성화 함수(주로 `tanh`)의 미분값은 최대 1입니다. 역전파 과정에서 1보다 작은 값이 계속 곱해지면, 그래디언트는 0에 수렴하게 됩니다. 결국, 먼 과거의 정보는 가중치 업데이트에 거의 영향을 주지 못합니다. 이것이 RNN이 긴 문장을 잘 기억하지 못하는 주된 이유입니다.

## 6. LSTM: 장기 기억을 위한 해법

**LSTM(Long Short-Term Memory)** 은 이 장기 의존성 문제를 해결하기 위해 고안된 RNN의 개선된 버전입니다. LSTM은 '단기 기억'과 '장기 기억'을 모두 효과적으로 다룰 수 있습니다.

LSTM의 핵심은 **셀 상태(Cell State)** 와 **게이트(Gate)** 라는 정교한 메커니즘입니다.

![LSTM Structure](https://i.imgur.com/0uTjG3E.png)
*(이미지 출처: Christopher Olah's Blog)*

- **셀 상태 (Cell State)**: 컨베이어 벨트처럼 네트워크 전체를 가로지르는 정보의 흐름입니다. LSTM은 이 셀 상태에 정보를 추가하거나 제거하는 능력을 가지고 있습니다.
- **게이트 (Gates)**: 정보가 셀 상태를 통과하도록 제어하는 장치입니다. 게이트는 시그모이드(Sigmoid) 함수를 통해 0(정보를 차단)과 1(정보를 통과) 사이의 값을 출력하여 정보의 양을 조절합니다.

LSTM에는 3가지 주요 게이트가 있습니다.

1.  **Forget Gate (망각 게이트)**: 과거의 정보 중 **무엇을 잊어버릴지** 결정합니다. "새로운 문장이 시작되었으니, 이전 문장의 주어는 잊어버리자."
2.  **Input Gate (입력 게이트)**: 새로운 정보 중 **무엇을 저장할지** 결정합니다. "새로운 주어 '그녀'가 등장했으니, 이 정보를 셀 상태에 추가하자."
3.  **Output Gate (출력 게이트)**: 셀 상태의 정보 중 **무엇을 출력으로 내보낼지** 결정합니다. "현재 시점에서는 주어 정보가 필요하니, '그녀'에 대한 정보를 출력하자."

이러한 게이트 구조 덕분에 LSTM은 중요한 정보는 오래 기억하고, 불필요한 정보는 잊어버릴 수 있어 RNN의 장기 의존성 문제를 효과적으로 해결합니다.

---

## 7. 직접 해보기 (Hands-on Lab): "hello" 예측하기

간단한 RNN 모델을 만들어, "hell" 이라는 입력을 받았을 때 마지막 글자 "o"를 예측하도록 학습시켜 보겠습니다.

```python
import torch
import torch.nn as nn
import numpy as np

# 1. 데이터 준비
# 입력 데이터: "hell" -> "o" 를 예측하는 것이 목표
# 어휘집(vocabulary) 생성
input_str = 'hello'
vocab = sorted(list(set(input_str)))
char_to_idx = {c: i for i, c in enumerate(vocab)}
idx_to_char = {i: c for i, c in enumerate(vocab)}

# 데이터셋 생성 (hell -> o)
input_data = [char_to_idx[c] for c in 'hell']
target_data = [char_to_idx['o']]

# 원-핫 인코딩
vocab_size = len(vocab)
x_one_hot = [np.eye(vocab_size)[x] for x in input_data]
y_one_hot = [np.eye(vocab_size)[y] for y in target_data]

# PyTorch 텐서로 변환 (sequence_length, batch_size, input_size)
inputs = torch.Tensor(x_one_hot).unsqueeze(1)
targets = torch.Tensor(y_one_hot).squeeze(0).argmax(dim=1)


# 2. RNN 모델 정의
class SimpleRNN(nn.Module):
    def __init__(self, input_size, hidden_size, output_size):
        super(SimpleRNN, self).__init__()
        self.hidden_size = hidden_size
        self.rnn = nn.RNN(input_size, hidden_size, batch_first=False) # batch_first=False 가 기본값
        self.fc = nn.Linear(hidden_size, output_size)

    def forward(self, x, hidden):
        # x: (sequence_length, batch_size, input_size)
        # hidden: (num_layers, batch_size, hidden_size)
        out, hidden = self.rnn(x, hidden)
        
        # 마지막 시퀀스의 출력만 사용
        # out: (sequence_length, batch_size, hidden_size)
        out = self.fc(out[-1, :, :]) # (batch_size, hidden_size) -> (batch_size, output_size)
        return out, hidden

    def init_hidden(self, batch_size=1):
        return torch.zeros(1, batch_size, self.hidden_size)

# 3. 모델, 손실함수, 옵티마이저 설정
input_size = vocab_size
hidden_size = 5
output_size = vocab_size
learning_rate = 0.1

model = SimpleRNN(input_size, hidden_size, output_size)
criterion = nn.CrossEntropyLoss()
optimizer = torch.optim.Adam(model.parameters(), lr=learning_rate)

# 4. 학습 루프
for epoch in range(100):
    hidden = model.init_hidden(batch_size=1)
    
    optimizer.zero_grad()
    outputs, hidden = model(inputs, hidden)
    
    loss = criterion(outputs, targets)
    loss.backward()
    optimizer.step()

    if (epoch + 1) % 10 == 0:
        _, predicted_idx = outputs.max(1)
        predicted_char = idx_to_char[predicted_idx.item()]
        print(f'Epoch [{epoch+1}/100], Loss: {loss.item():.4f}, Predicted: "{predicted_char}" -> Target: "o"')

# 5. 최종 예측
hidden = model.init_hidden(batch_size=1)
outputs, hidden = model(inputs, hidden)
_, predicted_idx = outputs.max(1)
predicted_char = idx_to_char[predicted_idx.item()]

print(f'\nFinal prediction after training: "hell" -> "{predicted_char}"')
```

> [!TIP]
> 위 코드를 직접 실행해보면서 `hidden_size`, `learning_rate`, `epoch` 등의 하이퍼파라미터를 바꿔보세요. `nn.RNN`을 `nn.LSTM`으로 바꾸고 실행해보는 것도 좋습니다. (LSTM은 `forward`에서 `hidden`이 `(h_0, c_0)` 튜플 형태여야 합니다)


## 8. 되짚어보기 (Summary)

- **RNN**: 순서가 있는 데이터를 다루기 위해, 네트워크의 출력이 다시 입력으로 들어가는 **순환** 구조를 가집니다. **은닉 상태**를 통해 과거의 정보를 기억합니다.
- **장기 의존성 문제**: RNN은 구조적 한계로 인해 먼 과거의 정보를 기억하기 어렵습니다 (**기울기 소실**).
- **LSTM**: **셀 상태**와 **게이트(망각, 입력, 출력)** 라는 정교한 메커니즘을 도입하여, 정보의 흐름을 효과적으로 제어하고 장기 기억을 가능하게 합니다.
- **활용**: RNN과 LSTM은 자연어 처리, 기계 번역, 시계열 예측 등 다양한 분야에서 딥러닝의 발전을 이끌고 있습니다.

## 9. 더 깊이 알아보기 (Further Reading)
- [The Unreasonable Effectiveness of Recurrent Neural Networks](http://karpathy.github.io/2015/05/21/rnn-effectiveness/): Andrej Karpathy의 전설적인 블로그 포스트. RNN의 매력적인 가능성을 보여줍니다.
- [Understanding LSTM Networks](http://colah.github.io/posts/2015-08-Understanding-LSTMs/): Christopher Olah의 블로그 포스트. LSTM의 구조를 그림으로 매우 알기 쉽게 설명한 최고의 자료입니다.

---

**➡️ 다음 시간: [Part 7.2: Transformer와 LLM의 핵심 원리](./part_7.2_transformer_and_llm_principles.md)** 